#!/bin/csh -f
# ************************************************************************
# *                                                                      *
# *   Usage: check_file.csh <file> <mode>                                *
# *                                                                      *
# *   C-shell program check_file.csh  checks existing fiule or a commend *
# *   in dependence of the second argument.                              *
# *                                                                      *
# *   1) file_read: Checks whether the file specified in the first       *
# *      argument exists and has a reading permission.                   *
# *                                                                      *
# *   2) file_execute: Checks whether the file specified in the first    *
# *      argument exists and has an execution permission.                *
# *                                                                      *
# *   3) command: Checks whether the command present in the system and   *
# *      can be executed.                                                *
# *                                                                      *
# *   check_file.csh  reports his activity at the screen. It returns     *
# *   status 0 in the case of success and 1 in the case of failure.      *
# *                                                                      *
# *  ### 02-JUN-2000 check_file.csh v1.0 (c) L. Petrov  02-JUN-2000 ###  *
# *                                                                      *
# ************************************************************************
#
if ( $1 == "" ) exit 0
switch ( $2 ) 
    case "file_read":
#
      echo -n "Checking file $1 ...\c"
      if ( -e $1 ) then
        else
           echo "  not found"
           exit (1)
      endif
      if ( -r $1) then
           echo "  ok"
           exit ( 0 )
         else 
           echo "  exist, but not readable"
           exit (1)
      endif
      breaksw
    case "file_execute":
#
      echo -n "Checking file $1 ...\c"
      if (-e $1) then
        else
           echo "  not found"
           exit (1)
      endif
      if ( -x $1) then
           echo "  ok"
           exit ( 0 )
         else 
           echo "  exist, but not executable"
           exit (1)
      endif
      breaksw
    case "command":
#
      echo -n "Checking command $1 ...\c"
      set which_answer = `which $1`
      if ( $#which_answer > 2 ) then
           if ( $which_answer[1] == "no" || $which_answer[2] == "$1" || $which_answer[3] == "in"  & ) then
                echo "  not found"
		exit(1)
           endif
      endif
#
      echo "  ok"
      exit ( 0 )
      breaksw
    case default:
      echo "check_file.csh: Wrong second argument $2"
      exit (-2)
endsw
