      PROGRAM    HW95_OUT
! $NOSTANDARD IO ON
! ************************************************************************
! *                                                                      *
! *     Program  HW95_OUT  reads the file of the coefficients of full      *
! *   harmonic development of tide-generating potential HW95             *
! *   (T. Hartmann and H-G. Wenzel "The HW95 tidal potential catalogue", *
! *   Geophysical Research Letters", Vol. 22(24), p. 3553-3556, 1995),   *
! *   chose terms of the 2-nd degree potential with amplitude higher of  *
! *   the certain truncation limit and forms the file with FORTRAN DATA  *
! *   structure for another subroutines which will calculate site        *
! *   displacements due to solid Earth tides of the 2-nd degree.         *
! *                                                                      *
! *     Amplitudes are renormalized! New normalization scheme is         *
! *   normalization over the unit sphere.                                *
! *                                                                      *
! *     If several records in the original HW95 file correspond to the   *
! *   same wave, they are combined in one record.                        *
! *                                                                      *
! *     Output file will contain phase, frequency, doubled angular       *
! *   acceleration for each tidal waves included, as well as amplitude   *
! *   and order of the harmonic function for the waves with amplitudes   *
! *   larger than the specified limit.                                   *
! *                                                                      *
! *   /tmp/hw95_unsorted.waves  -- unsorted list of waves. Their order   *
! *                                is in increasing of HW95 numbers.     *
! *   /tmp/hw95_sorted.waves    -- sorted list of waves. They are put    *
! *                                in the order of decreasing their      *
! *                                amplitudes.                           *
! *   /tmp/hw95_norm.inc        -- include file to be added to the       *
! *                                Fortran program using restricted HW95 *
! *                                catalogue of the tidal vales.         *
! *                                                                      *
! *   The input HW95 is in the file specified in the variable  HW_FILE   *
! *                                                                      *
! *  ###  27-JUN-1997   HW95_OUT   v3.1  (c)  L. Petrov 17-JUL-2002 ###  *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE
      INTEGER*4  MW, NW, IUER
      PARAMETER (MW=16384)
      CHARACTER  BUF(MW)*128, LAST_BUF*128, DAR*4, STR*128, STR1*128, OUT*80
      CHARACTER  COMM*1, CONT*1, PREF*8, PSTF*4
      INCLUDE   'hw95_out.i'
      INTEGER*4  IPH, KW, J0, J1, J2, J3, J4, NW_NEW, IER
      INTEGER*4  L, M, K(11), ID, IP, IR, IPRF, IPST
      REAL*8     S0, C0, S1, C1, FDP, CURR_AMPL, LAST_AMPL, AMPL_LIM
      INTEGER*4  KL, KH
      REAL*8     GM, REA, G_ACC, H2, L2, H3, L3
      PARAMETER ( GM  = 3.986004418D14 )  !  Geocentric gravitational constant
      PARAMETER ( REA = 6378136.3D0    )  !  Equatorial radius
      PARAMETER ( G_ACC = GM / REA**2  )  !  Equatorial gravity accelaration
      PARAMETER ( H2 = 0.609D0         )  !  Nominal h2 Love number
      PARAMETER ( L2 = 0.0852D0        )  !  Nominal l2 Love number
      PARAMETER ( H3 = 0.2920D0        )  !  Nominal h3 Love number
      PARAMETER ( L3 = 0.0152D0        )  !  Nominal l3 Love number
      INTEGER*4  L_TARGET
      REAL*8     NC(0:3,2:3), NUC(0:3)
      REAL*8     SIMON_ING(11,5), SIMON_TAB(3,11)
      INTEGER*4  N1, N2, NUMARG
      REAL*8     PI, PI2, P2I, YS
      REAL*8     AMP, PHAS, FREQ, ACCL, PHAS_NEW
      REAL*8     SOTID_ATAN_SC
      LOGICAL    LEX, FL_HPUX, FL_SUNOS
      INTEGER*4, EXTERNAL :: I_LEN, ILEN, LINDEX
      PARAMETER ( PI=3.141592653589793D0, PI2=2.D0*PI, P2I=PI/2D0 ) !
      PARAMETER ( YS = 86400.D0 * 365.25D0 ) ! The number of seconds in Julian
!                                            ! centure
      INTEGER*4  IARGC 
!
! --- Ingenious table of Simon coeffitients used in HW95 potential
!
      DATA ( (SIMON_ING(N1,N2), N1=1,11), N2=1,5) &
!
! ----- Constant terms on the J2000 epoch in degrees
!
     &  / &
     &  242.14980452999D0, &
     &  218.31664562999D0, &
     &  280.46645016002D0, &
     &   83.35324311998D0, &
     &  234.95544499000D0, &
     &  282.93734098001D0, &
     &  252.25090551999D0, &
     &  181.97980085000D0, &
     &  355.43299958002D0, &
     &   34.35151874003D0, &
     &   50.07744430000D0, &
!
! ----- Rate of change in degrees per 1000 years
!
     &  127037328.88553056D0, &
     &    4812678.81195750D0, &
     &     360007.69748806D0, &
     &      40690.13635250D0, &
     &      19341.36261972D0, &
     &         17.19457667D0, &
     &    1494740.72172233D0, &
     &     585192.12953330D0, &
     &     191416.96370297D0, &
     &      30363.02774848D0, &
     &      12235.11068622D0, &
!
! ----- Second time derivative
!
     &   0.17696111D0, &
     &  -0.14663889D0, &
     &   0.03032222D0, &
     &  -1.03217222D0, &
     &  -0.20756111D0, &
     &   0.04568889D0, &
     &   0.03034984D0, &
     &   0.03101395D0, &
     &   0.03105187D0, &
     &   0.02232972D0, &
     &   0.05190783D0, &
!
! ----- Third time derivative
!
     &  -0.00183140D0, &
     &   0.00185140D0, &
     &   0.00002000D0, &
     &  -0.01249168D0, &
     &  -0.00213942D0, &
     &  -0.00001776D0, &
     &   0.00001811D0, &
     &   0.00001490D0, &
     &   0.00001564D0, &
     &   0.00003701D0, &
     &  -0.00002985D0, &
!
! ----- Fourth time derivative
!
     &   0.00008824D0, &
     &  -0.00015355D0, &
     &  -0.00006532D0, &
     &   0.00052655D0, &
     &   0.00016501D0, &
     &  -0.00003323D0, &
     &  -0.00006532D0, &
     &  -0.00006532D0, &
     &  -0.00006532D0, &
     &  -0.00005214D0, &
     &  -0.00009740D0 &
     &   /
!
      FL_HPUX  = .FALSE.
      FL_SUNOS = .FALSE.
#ifdef HPUX
      FL_HPUX = .TRUE.
#endif     
#ifdef SUNOS
      FL_SUNOS = .TRUE.
#endif     

      NUMARG = IARGC ()
      IF ( NUMARG .GE. 2 ) THEN
           IF ( FL_HPUX ) THEN
                CALL GETARG ( 1+1, STR1 )
                CALL GETARG ( 1+2, STR  )
              ELSE 
                CALL GETARG ( 1, STR1 )
                CALL GETARG ( 2, STR  )
           END IF
        ELSE
           WRITE ( 6, '(A)' ) 'hw95_out v3.0  Usage: hw96_out <degree> '// &
     &                        '<ampl_lim>'
           CALL EXIT ( 1 )
      END IF
!
      CALL CHASHL ( STR  )
      CALL CHASHL ( STR1 )
      IF ( STR1(1:1) .EQ. '2' ) THEN
           L_TARGET = 2
         ELSE IF ( STR1(1:1) .EQ. '3' ) THEN
           L_TARGET = 3
         ELSE
           WRITE ( 6, * ) 'Only degree 2 or 3 is supported, not '// &
     &                    STR(1:I_LEN(STR1))
          CALL EXIT ( 1 )
      END IF
      IF ( INDEX ( STR, '.' ) .EQ. 0 ) STR = STR(1:I_LEN(STR))//'.'
      READ ( UNIT=STR, FMT='(F9.0)' ) AMPL_LIM
      WRITE ( 6, * ) ' Potential of the ', L_TARGET, ' degree'
      WRITE ( 6, * ) ' Amplitude  limit ', AMPL_LIM, ' was chosen'
!
      IF ( HW_INC_FORMAT .EQ. 'f77' ) THEN
           COMM = 'C'
           CONT = ' '
           PREF = '     # '
           IPRF = 7
           PSTF = ', '
           IPST = 2
         ELSE
           COMM = '!'
           CONT = '&'
           PREF = '   & '
           IPRF = 5
           PSTF = ', & '
           IPST = 4
       END IF
!
! --- Transformation of original Simon coeffitients to rad, rad/sec and
! --- rad/sec**2 (highers derivatives can be neglected)
!
      DO 400 J0=1,11
         SIMON_TAB(1,J0) = SIMON_ING(J0,1)*(PI/180.D0)
         SIMON_TAB(2,J0) = SIMON_ING(J0,2)*(PI/180.D0)/(1000.D0*YS)
         SIMON_TAB(3,J0) = SIMON_ING(J0,3)*(PI/180.D0)/(1000.D0*YS)**2
 400  CONTINUE
!
! --- Normalization coeffitients
!
      NC(0,2) = DSQRT(   5.D0      )   ! normalizing to have the max value 1
      NC(1,2) = DSQRT(  15.D0/4.D0 )
      NC(2,2) = DSQRT(  15.D0/4.D0 )
!
      NC(0,3) = DSQRT(   7.D0       )  ! normalizing to have the max value 1
      NC(1,3) = DSQRT(  21.D0/2.D0  )
      NC(2,3) = DSQRT( 105.D0/4.D0  )
      NC(3,3) = DSQRT(  35.D0/8.D0  )
!
      NUC(0) = DSQRT ( 4.0D0 * PI )   ! Normalization over the unit sphere
      NUC(1) = DSQRT ( 8.0D0 * PI )
      NUC(2) = DSQRT ( 8.0D0 * PI )
      NUC(3) = DSQRT ( 8.0D0 * PI )
!
! --- Reading original file with development HW95
!
      IUER = -1
      CALL SOTID_RD_TEXT ( HW_FILE, MW, BUF, NW, IUER )
      IF ( IUER .NE. 0 ) CALL EXIT ( 1 )
!
! --- removing temporary files
!
      INQUIRE ( FILE=HW1_OUTFIL, EXIST=LEX )
      IF ( LEX ) CALL SYSTEM ( 'rm '//HW1_OUTFIL )
      INQUIRE ( FILE=HW2_OUTFIL, EXIST=LEX )
      IF ( LEX ) CALL SYSTEM ( 'rm '//HW2_OUTFIL )
      INQUIRE ( FILE=HW3_OUTFIL, EXIST=LEX )
      IF ( LEX ) CALL SYSTEM ( 'rm '//HW3_OUTFIL )
!
! --- Openning the first output temporary file
!
      OPEN ( UNIT=16, FILE=HW1_OUTFIL, STATUS='NEW' )
!
      IPH = 0
      KW = 0
      KL = 0
      KH = 0
      DO 410 J1=1,NW  ! cycle on lines of HW95 file
         IF ( BUF(J1)(1:10) .EQ. 'C*********' ) THEN
!
! ----------- We should skip all lines before line beginning with C****
!
              IPH = 1
              GOTO 410
         END IF
         IF ( BUF(J1)(1:6) .EQ. '999999' ) IPH = 2 ! attribute the end of file
         IF ( IPH .EQ. 1 ) THEN
              KW = KW+1
!
! ----------- Decoding fields of the line of the HW95 file
!
              READ ( UNIT=BUF(J1)(10:11), FMT='(I2)' ) L
              READ ( UNIT=BUF(J1)(12:14), FMT='(I3)' ) M
              K(1) = M
              READ ( UNIT=BUF(J1)(15:17), FMT='(I3)' ) K(2)
              READ ( UNIT=BUF(J1)(18:20), FMT='(I3)' ) K(3)
              READ ( UNIT=BUF(J1)(21:23), FMT='(I3)' ) K(4)
              READ ( UNIT=BUF(J1)(24:26), FMT='(I3)' ) K(5)
              READ ( UNIT=BUF(J1)(27:29), FMT='(I3)' ) K(6)
              READ ( UNIT=BUF(J1)(30:32), FMT='(I3)' ) K(7)
              READ ( UNIT=BUF(J1)(33:35), FMT='(I3)' ) K(8)
              READ ( UNIT=BUF(J1)(36:38), FMT='(I3)' ) K(9)
              READ ( UNIT=BUF(J1)(39:41), FMT='(I3)' ) K(10)
              READ ( UNIT=BUF(J1)(42:44), FMT='(I3)' ) K(11)
!
              READ ( UNIT=BUF(J1)(45:56), FMT='(F12.0)' ) FDP
              READ ( UNIT=BUF(J1)(57:68), FMT='(F12.0)' ) C0
              READ ( UNIT=BUF(J1)(69:80), FMT='(F12.0)' ) S0
              READ ( UNIT=BUF(J1)(81:90), FMT='(F12.0)' ) C1
              READ ( UNIT=BUF(J1)(91:100),FMT='(F12.0)' ) S1
!
              DAR = BUF(J1)(102:105) ! Darwin symbol of the wave
              IF ( L .NE. L_TARGET  ) GOTO 410
              KL = KL + 1
!
! ----------- Transformation amplitude to m^2/sec^2 and velocities to m^2/sec^3
!
              C0 = C0*1.D-10
              S0 = S0*1.D-10
              C1 = C1*1.D-10/(YS*100.D0)
              S1 = S1*1.D-10/(YS*100.D0)
!
! ----------- Calculation amplitude of the potential (under assumption that
! ----------- Legendre polynomials are normalised to have maximum values
! ----------- to be 1)
!
              AMP = NUC(M) * SQRT ( C0**2 + S0**2 )
!
! ----------- Calculation max possible amplitude of the tide for this wave
! ----------- (under consumption that the station located on the latitude where
! ----------- Legendre polynomial has maximum).
!
              IF ( L .EQ. L_TARGET .AND.  AMP .GT. AMPL_LIM ) THEN
!
! ---------------- This the tide of the 2-nd degree and the amplitude of this
! ---------------- wave is above the truncation level
!
                   KH   = KH + 1  ! Increment counter of the appropriate waves
!
! ---------------- Transformation from the representation of the potential
! ---------------- in the form (C0 + C1*t)*COS ARG + (S0 + S1*t)*SIN ARG
! ---------------- to the form: AMP * COS ( ACCL*t**2 + FREQ*t + PHAS )
!
! ---------------- Firstly, we calculate initial phase and frequency of the
! ---------------- tidal wave.
!
                   PHAS = -SOTID_ATAN_SC ( S0, C0 )
                   FREQ =  0.0D0
                   ACCL =  0.0D0
!
! ---------------- Now we calculate phase, frequency and doubled angular
! ---------------- acceleration of the tidal wave
!
                   DO 420 J2=1,11
                      PHAS = PHAS + SIMON_TAB(1,J2)*K(J2)
                      FREQ = FREQ + SIMON_TAB(2,J2)*K(J2)
                      ACCL = ACCL + 2.D0*SIMON_TAB(3,J2)*K(J2)
 420               CONTINUE
!
! ---------------- Eliminate 2pi redundancy from the phase
!
                   ID = IDINT ( PHAS/PI2 )
                   IF ( PHAS .LT. 0.D0 ) ID=ID-1
                   PHAS = PHAS - ID*PI2
!
! ---------------- And write the line to the output file
!
                   IF ( FL_SUNOS ) THEN
!
! --------------------- Hmm. Presumably SunOS Fortran has a bug. It process
! --------------------- incorrectly fortran I/O specification. Try to work
! --------------------- around this bug
!
                        WRITE  ( UNIT=OUT, FMT=120 ) PREF(1:IPRF), PHAS, FREQ, &
     &                           ACCL, AMP, M, PSTF(1:IPST), KW, DAR, 0
                        IP = INDEX ( OUT, " ." ) 
                        IF ( IP .GT. 0 ) OUT = OUT(1:IP)//'0'//OUT(IP+1:)
                        IP = INDEX ( OUT, " ." ) 
                        IF ( IP .GT. 0 ) OUT = OUT(1:IP)//'0'//OUT(IP+1:)
                        IP = INDEX ( OUT, " ." ) 
                        IF ( IP .GT. 0 ) OUT = OUT(1:IP)//'0'//OUT(IP+1:)
                        WRITE  ( UNIT=16, FMT='(A)' ) OUT(1:I_LEN(OUT))
                     ELSE
                   WRITE  ( 16, 120 ) PREF(1:IPRF), PHAS, FREQ, ACCL, AMP, M, &
     &                                PSTF(1:IPST), KW, DAR, 0
                   END IF
 120               FORMAT ( A, F8.6, 'D0, ', 1PD18.12, ', ', 1PD9.2, ', ', &
     &                      0PF9.7, ', ', I1, A, '! ', I5, ' ', A4, I3 )
!
! ---------------- Now we try to approximate mixed terms:
! ---------------- C1*t*COS(OM1*t+PHI1) = (approx)
! ----------------      C1/(2*OM0)*COS((OM1+OM0)*t + PHI1 - PI/2) +
! ----------------      C1/(2*OM0)*COS((OM1-OM0)*t + PHI1 + PI/2)
! ---------------- S1*t*SIN(OM1*t+PHI1) = (approx)
! ----------------      S1/(2*OM0)*COS((OM1+OM0)*t + PHI1 + PI) +
! ----------------      S1/(2*OM0)*COS((OM1-OM0)*t + PHI1)
! ----------------
! ---------------- where OM0 -- frequency resolution (usually very small)
!
                   IF ( NUC(M)*C1/(2.D0*FREQ_RES) .GT. AMPL_LIM ) THEN
                        KW = KW+1
                        PHAS_NEW = PHAS - P2I
                        IF ( PHAS_NEW .LT. 0.0D0 ) PHAS_NEW = PHAS_NEW + PI2
                        WRITE  ( 16, 120 ) PREF(1:IPRF), PHAS_NEW, &
     &                           FREQ+FREQ_RES, ACCL, NUC(M)*C1/(2.D0*FREQ_RES), &
     &                           M, PSTF(1:IPST), KW, DAR, 1
!
                        KW = KW+1
                        PHAS_NEW = PHAS + P2I
                        IF ( PHAS_NEW .GT. PI2 ) PHAS_NEW = PHAS_NEW - PI2
                        WRITE  ( 16, 120 ) PREF(1:IPRF), PHAS_NEW, &
     &                           FREQ-FREQ_RES, ACCL, NUC(M)*C1/(2.D0*FREQ_RES), &
     &                           M, PSTF(1:IPST), KW, DAR, 2
                   END IF
!
                   IF ( NUC(M)*S1/(2.D0*FREQ_RES) .GT. AMPL_LIM  ) THEN
                        KW = KW+1
                        PHAS_NEW = PHAS + PI
                        IF ( PHAS_NEW .GT. PI2 ) PHAS_NEW = PHAS_NEW - PI2
                        WRITE  ( 16, 120 ) PREF(1:IPRF), PHAS_NEW, &
     &                           FREQ+FREQ_RES, ACCL, NUC(M)*S1/(2.D0*FREQ_RES), &
     &                           M, PSTF(1:IPST), KW, DAR, 1
!
                        KW = KW+1
                        PHAS_NEW = PHAS
                        WRITE  ( 16, 120 ) PREF(1:IPRF), PHAS_NEW, &
     &                           FREQ-FREQ_RES, ACCL, NUC(M)*S1/(2.D0*FREQ_RES), &
     &                           M, PSTF(1:IPST), KW, DAR, 2
                   END IF
              END IF
         END IF
 410  CONTINUE
      CLOSE ( UNIT = 16 )
!
! --- Now sort output temporary file to make it in the order of increasing
! --- their frequencies.
!
      IF ( HW_INC_FORMAT .EQ. 'f77' ) THEN
           CALL SYSTEM ( 'sort -k 1.35,1.37n -k 1.20,1.33n '// &
     &                    HW1_OUTFIL(1:I_LEN(HW1_OUTFIL))// &
     &                    ' > '//HW2_OUTFIL(1:I_LEN(HW2_OUTFIL))//CHAR(0) )
         ELSE
           CALL SYSTEM ( 'sort -k 1.33,1.35n -k 1.18,1.31n '// &
     &                    HW1_OUTFIL(1:I_LEN(HW1_OUTFIL))// &
     &                    ' > '//HW2_OUTFIL(1:I_LEN(HW2_OUTFIL))//CHAR(0) )
      END IF
!
      WRITE ( 6, * ) ' Total number of waves read:          ', KW
      WRITE ( 6, * ) ' Number of waves of 2-nd degree read: ', KL
      WRITE ( 6, * ) ' Number of waves selected written:    ', KH
!
! --- Read temporary file
!
      IUER = -1
      CALL SOTID_RD_TEXT ( HW2_OUTFIL, MW, BUF, NW, IUER )
      IF ( IUER .NE. 0 ) CALL EXIT ( 1 )
!
! --- Open the output file
!
      OPEN ( UNIT=16, FILE=HW1_OUTFIL, STATUS='UNKNOWN' )
!
! --- The purpose of this code is to squeeze several consectuve records.
! --- Unfotunately, some tidal waves have TWO records: due to the Moon and
! --- due to the Sun. They have the same arguments but different amplitudes.
! --- We sum amplitdes of the identical waves and write down the sum.
!
      NW_NEW = 0
      CALL CLRCH ( LAST_BUF )
      IF ( HW_INC_FORMAT .EQ. 'f77' ) THEN
           IR = 49 ! column of the last character of the end of tide
!                  ! accelration part
         ELSE
           IR = 47
      END IF
      DO 430 J3=1,NW
         IF ( BUF(J3)(1:IR) .EQ. LAST_BUF(1:IR) ) THEN
              READ  ( LAST_BUF(IR+2:IR+10), FMT='(F9.0)' ) LAST_AMPL
              READ  (  BUF(J3)(IR+2:IR+10), FMT='(F9.0)' ) CURR_AMPL
              IF ( CURR_AMPL .GT. LAST_AMPL    ) LAST_BUF(67:71)=BUF(J3)(67:71)
              IF ( LAST_BUF(73:76) .EQ. '    ' ) LAST_BUF(73:76)=BUF(J3)(73:76)
              WRITE ( LAST_BUF(IR+2:IR+10), FMT='(F9.7)' ) LAST_AMPL+CURR_AMPL
              IP = INDEX ( LAST_BUF, '!' ) + 1
              IF ( IP .GT. 1 ) LAST_BUF(IP:IP) = '*'
            ELSE
              NW_NEW = NW_NEW + 1
              IF ( ILEN(LAST_BUF) .GT. 0 ) WRITE ( 16, FMT='(A)' ) LAST_BUF
              LAST_BUF = BUF(J3)
         END IF
 430  CONTINUE
      WRITE ( 16, FMT='(A)' ) LAST_BUF
      CLOSE ( UNIT=16 )
      NW = NW_NEW
!
! --- Now sort output temporary file to make it in the order of decreasing
! --- of the amplitudes.
!
      CALL SYSTEM ( 'sort -k 5nr '// &
     &              HW1_OUTFIL(1:I_LEN(HW1_OUTFIL))// &
     &              ' > '//HW2_OUTFIL(1:I_LEN(HW2_OUTFIL)) )
!
! --- Now again open temporary file for read...
!
      IUER = -1
      CALL SOTID_RD_TEXT ( HW2_OUTFIL, MW, BUF, NW, IUER )
      IF ( IUER .NE. 0 ) CALL EXIT ( 1 )
!
! --- And eventuial output file -- for write
!
      OPEN ( UNIT=16, FILE=HW3_OUTFIL, STATUS='NEW' )
      WRITE ( 16, FMT='(A)' ) COMM
      WRITE ( 16, FMT='(A)' ) COMM//' --- This file was generated '// &
     &                        'automatically by program hw95_out v 3.0'
      WRITE ( 16, FMT='(A)' ) COMM//' --- written by Leonid Petrov '// &
     &                        '(pet@leo.gsfc.nasa.gov)'
      WRITE ( 16, FMT='(A)' ) COMM//' --- It contains phases, frequencies, '// &
     &                        'accelerations amplitudes and '
      WRITE ( 16, FMT='(A,I1,A)' ) COMM//' --- orders of the tidal waves '// &
     &                            'of the ', L_TARGET, ' degree '
      WRITE ( 16, FMT='(A,1PD10.3,A)' ) COMM//' --- with amplitudes greater '// &
     &                                 'than ', AMPL_LIM, ' m^2/sec^2'
      WRITE ( 16, FMT='(A)' ) COMM//' --- Harmonic expansion HW95 was used.'
      WRITE ( 16, FMT='(A)' ) COMM//' --- (T. Hartmann and H-G. Wenzel "The '// &
     &                        'HW95 tidal potential catalogue",'
      WRITE ( 16, FMT='(A)' ) COMM//' ---  Geophysical Research Letters", '// &
     &                        'Vol. 22(24), p. 3553-3556, 1995),'
      WRITE ( 16, FMT='(A)' ) COMM//' ---  Amplitudes were renormalized!'
      WRITE ( 16, FMT='(A)' ) COMM
!
! --- Writing declarations
!
      CALL CLRCH ( OUT )
      IF ( L_TARGET .EQ. 2 ) THEN
           OUT = '      INTEGER*4   NW, N2'
         ELSE IF ( L_TARGET .EQ. 3 ) THEN
           OUT = '      INTEGER*4   NW3, N3'
      END IF
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
      CALL CLRCH ( STR )
      WRITE ( UNIT=STR, FMT='(I6)' ) NW
      CALL CHASHL ( STR )
      CALL CLRCH ( OUT )
      IF ( L_TARGET .EQ. 2 ) THEN
           OUT='      PARAMETER ( NW = '//STR(1:I_LEN(STR))//' )'
         ELSE IF ( L_TARGET .EQ. 3 ) THEN
           OUT='      PARAMETER ( NW3 = '//STR(1:I_LEN(STR))//' )'
      END IF
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
      CALL CLRCH ( OUT )
      IF ( L_TARGET .EQ. 2 ) THEN
           OUT ='      REAL*8      PHAS(NW), FREQ(NW), ACCL(NW), AMPL(NW)'
         ELSE IF ( L_TARGET .EQ. 3 ) THEN
           OUT ='      REAL*8      PHAS3(NW3), FREQ3(NW3), ACCL3(NW3), '// &
     &          'AMPL3(NW3)'
      END IF
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
      CALL CLRCH ( OUT )
      IF ( L_TARGET .EQ. 2 ) THEN
           OUT ='      INTEGER*4   M(NW)'
         ELSE IF ( L_TARGET .EQ. 3 ) THEN
           OUT ='      INTEGER*4   M3(NW3)'
      END IF
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
      CALL CLRCH ( STR      )
      STR(1:1) = '1'
      CALL CLRCH ( STR1     )
      STR1 = '10'
      CALL CLRCH ( OUT )
      IF ( L_TARGET .EQ. 2 ) THEN
          OUT ='      DATA      ( PHAS(N2), FREQ(N2), ACCL(N2), AMPL(N2), '// &
     &         'M(N2), N2='//STR(1:I_LEN(STR))//','//STR1(1:I_LEN(STR1))//')'// &
     &         ' '//CONT
         ELSE IF ( L_TARGET .EQ. 3 ) THEN
          OUT ='      DATA ( PHAS3(N3), FREQ3(N3), ACCL3(N3), AMPL3(N3), '// &
     &         'M3(N3), N3='//STR(1:I_LEN(STR))//','//STR1(1:I_LEN(STR1))//')'// &
     &         ' '//CONT
      END IF
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
      CALL CLRCH ( OUT )
      OUT = PREF(1:IPRF)//'/ '//CONT
      WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
! --- Writing coeffitients itself
!
      DO 440 J4=1,NW
         CALL CLRCH    ( OUT )
         OUT = BUF(J4)(1:77)
         IF ( MOD(J4,10) .EQ. 0  .OR.  J4 .EQ. NW ) THEN
!
! ----------- Remove the comma
!
              IP = LINDEX ( OUT, ',' )
              IF ( IP .GT. 0 ) OUT(IP:IP) = ' '
         END IF
!
! ------ We add number of the wave in this file to the comment field
!
         IF ( J4 .LT. 10000 ) THEN
              WRITE ( UNIT=OUT(76:80), FMT='(I4)' ) J4
              CALL CHASHR (     OUT(77:80) )
            ELSE
              WRITE ( UNIT=OUT(76:80), FMT='(I5)' ) J4
              CALL CHASHR (     OUT(76:80) )
         END IF
         WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
!
! ------ And each 10 lines we interrupt data structure. The true is that some
! ------ compilers cannot endure more than some limit of consequet contination
! ------ lines
!
         IF ( MOD(J4,10) .EQ. 0   .AND.  J4 .NE. NW ) THEN
              WRITE ( 16, FMT='(A)' ) PREF(1:IPRF)//'/'
              WRITE ( 16, FMT='(A)' ) COMM
              CALL CLRCH ( STR      )
              WRITE ( UNIT=STR, FMT='(I6)' ) J4+1
              CALL CHASHL ( STR )
              CALL CLRCH ( STR1 )
              WRITE ( UNIT=STR1, FMT='(I6)' ) MIN((J4+10),NW)
              CALL CHASHL ( STR1 )
              CALL CLRCH ( OUT )
              IF ( L_TARGET .EQ. 2 ) THEN
                   OUT ='      DATA      ( PHAS(N2), FREQ(N2), ACCL(N2), '// &
     &                'AMPL(N2), M(N2), '// &
     &                ' N2='//STR(1:I_LEN(STR))//','//STR1(1:I_LEN(STR1))//')'// &
     &                ' '//CONT
                 ELSE IF ( L_TARGET .EQ. 3 ) THEN
                   OUT ='      DATA ( PHAS3(N3), FREQ3(N3), ACCL3(N3), '// &
     &                'AMPL3(N3), M3(N3), '// &
     &                'N3='//STR(1:I_LEN(STR))//','//STR1(1:I_LEN(STR1))//')'// &
     &                ' '//CONT
              END IF
              WRITE ( 16, FMT='(A)' ) OUT(1:I_LEN(OUT))
              WRITE ( 16, FMT='(A)' ) PREF(1:IPRF)//'/ '//CONT
         END IF
 440  CONTINUE
!
      WRITE ( 16, FMT='(A)' ) PREF(1:IPRF)//'/'
!
! --- Final close. Deal done
!
      CLOSE ( UNIT = 16 )
!
      WRITE ( 6, * ) ' Output unsorted list of waves: '// &
     &                 HW1_OUTFIL(1:I_LEN(HW1_OUTFIL))
      WRITE ( 6, * ) ' Output sorted list of waves:   '// &
     &                 HW2_OUTFIL(1:I_LEN(HW2_OUTFIL))
      WRITE ( 6, * ) ' Output include files:          '// &
     &                 HW3_OUTFIL(1:I_LEN(HW3_OUTFIL))
!
      END  PROGRAM  HW95_OUT
