/*************************************************************************/
/*                                                                       */
/*   This test program provides an example of how to use sotid library   */
/*   from C. Site displacements due to solid Earth tides are computed    */
/*   for two epochs for five sites.                                      */
/*                                                                       */
/*  ### 16-JUL-2002  sotid_c_test  v1.0 (c) L. Petrov  16-JUL-2002 ###   */
/*                                                                       */
/*************************************************************************/

#include "sotid_type.h"
#define M_STA 5


main (argc, argv)
int argc;
char **argv;

{
    struct STATID__STRU STATID[M_STA] ;
    struct TIMTID__STRU TIMTID ;
    struct TIDCNF__STRU TIDCNF ; 

    #include "sotid_data.h"
    #include "sotid_functions.h"

    long   IUER ;
    long CNF_VAL;
    char   C_STA[M_STA][SOTID__NAM_LEN] ;
    double R_STA[M_STA][3] ;
    long   L_STA, J1, J2, M_TST, IS;
    long   ID, MJD, MJD_BEG ;  /* 2000.10.31 */
    char   STR[80] ;
    double TAI, TAI_BEG, DELTA_TAI, UT1_M_TAI, D_REN[3];

/*  
! --- Set coordinats and names of 5 stations
*/
      strncpy ( &C_STA[0][0], "Wz      ", SOTID__NAM_LEN );
      R_STA[0][0] = 4075539.895;
      R_STA[0][1] =  931735.270;
      R_STA[0][2] = 4801629.355;

      strncpy ( &C_STA[1][0], "Ny      ", SOTID__NAM_LEN );
      R_STA[1][0] = 1202462.761;
      R_STA[1][1] = 252734.404;
      R_STA[1][2] = 6237766.013;

      strncpy ( &C_STA[2][0], "Ap      ", SOTID__NAM_LEN );
      R_STA[2][0] =   918034.750;
      R_STA[2][1] = -4346132.269;
      R_STA[2][2] =  4561971.156;

      strncpy ( &C_STA[3][0], "Ts      ", SOTID__NAM_LEN ); /*  ! Tsukuba, Japan */
      R_STA[3][0] = -3957408.308;
      R_STA[3][1] =  3310229.259;
      R_STA[3][2] =  3737494.482;

      strncpy ( &C_STA[4][0], "Hh      ", SOTID__NAM_LEN ); /* Hartebeesthoek, South Africa */
      R_STA[4][0] =  5085442.796;
      R_STA[4][1] =  2668263.498;
      R_STA[4][2] = -2768697.043;
/*  
! --- Set sotid configuration 
*/
      IUER = -1;
      SOTID_SET ( &SOTID__GEN_ALL, &SOTID__MAT00, &SOTID__2D_012ORD, 
                  &SOTID__ZF_LOVE, &SOTID__3D_MDG97, &TIDCNF, &IUER );
      if ( IUER != 0 ) exit ( 1 );

      L_STA = M_STA ;
/*  
! --- Compute intermediary parameters which are independent on time,
! --- but depends only on site coordinates
*/
      IUER = -1 ;
      SOTID_PRE ( &L_STA, &C_STA, &R_STA, &TIDCNF, &STATID, &IUER, 
                  SOTID__NAM_LEN ) ;
      if ( IUER != 0 ) exit ( 1 ) ;
/*  
! --- Inquire information about current sotid settings and print this
! --- information in the screen
*/
      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_GEN_LOVE, &TIDCNF, &STR, &IS, &IUER, 
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_MODEL_2D, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_ORDER_2D, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_ZF_LOVE, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_MODEL_3D, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_N_STA, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_NW_D2, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

      IUER = -1;
      CNF_VAL = SOTID_INQ ( &SOTID__REQ_NW_D3, &TIDCNF, &STR, &IS, &IUER,
                            sizeof(STR) );
      if ( IUER != 0 ) exit ( 1 ) ;
      STR[IS] = 0; printf ( " %s \n", &STR );

/*  
! --- Set date and time 
*/
      M_TST     = 2 ;
      MJD_BEG   = 51848 ;  /* 2000.10.31 */
      TAI_BEG   = 65326.0 ;
      DELTA_TAI = 1500.0 ;
      UT1_M_TAI = -31.855734 ; 

      for ( J1=0; J1 < M_TST; J1++)
	{
          TAI = TAI_BEG + DELTA_TAI*J1;
          MJD = MJD_BEG;
          ID  = ( TAI/86400.0 );
          TAI = TAI - ID*86400.0;
          MJD = MJD + ID;
/*
! ------ Compute time-dependent intermediary quantities
*/
          IUER = -1;
          SOTID_TIM ( &MJD, &TAI, &UT1_M_TAI, &TIDCNF, &TIMTID, &IUER ) ;
          if ( IUER != 0 ) exit ( 1 ) ;
          printf ( "---------------|----|-------------------------------|\n");

          for ( J2=0; J2 < M_STA; J2++)
	    {
/*
! --------- Compute vector of site displacement for the J2-th station
*/
               SOTID_DSP ( &TIDCNF, &TIMTID, &STATID[J2], &D_REN );
               printf ( " %.5d %7.1f | %.2s | %8.5f  %8.5f  %8.5f  |\n", 
                        MJD, TAI, &C_STA[J2][0], 
                        D_REN[0], D_REN[1], D_REN[2] ) ;
            }
        }
       printf ( "----------------------------------------------------|\n");
       exit ( 0 );
}
