        SUBROUTINE SOTID_ERR_LOG ( NERR, IUER, PROG, MES )
! ************************************************************************
! *                                                                      *
! *     Routine SOTID_ERR_LOG  is the universal errors handler. If the   *
! *   input value of IUER = -1, then SOTID_ERR_LOG works in verbose      *
! *   mode, otherwise it works in silent mode.                           *
! *                                                                      *
! *     If NERR=0, it means that no errors occurred. In that case        *
! *   SOTID_ERR_LOG checks whether IUER is accessible for writing.       *
! *   It yes, it assigns IUER value 0 and returns.                       *
! *                                                                      *
! *     If NERR is not 0, it means that the error occurred.              *
! *   SOTID_ERR_LOG formats the line with error message. If the input    *
! *   value of IUER = -1, then it prints the error message in screen.    *
! *   If the input value of IUER is not -1, then SOTID_ERR_LOG does not  *
! *   print the message. Then SOTID_ERR_LOG checks whether IUER is       *
! *   accessible for writing. It yes, it assigns IUER value NERR and     *
! *   returns.                                                           *
! *                                                                      *
! *     SOTID_ERR_LOG formats the error message in following way:        *
! *   three dollars (hint that errors cost $), two blanks, IUER=xxxx     *
! *   (where xxxx is the value of NERR), one blank, pppppp -- value of   *
! *   argument PROG_NAME, then blank, double quote, message supplied     *
! *   by the argument MES, double quote, blank three dollar characters.  *
! *   example:                                                           *
! *                                                                      *
! *   $$$  IUER=xxxx ppppppp "mmmmmmmmmmmmmmm" $$$                       *
! *                                                                      *
! *   The portion of the error message "IUER=xxxx ppppppp" is printed    *
! *   in inverse mode and "mmmmmmmmmmmmm" message is printed by red      *
! *   color if the terminal can support inversion and colors.            *
! *                                                                      *
! *   If the total length of the formatted message exceeds the current   *
! *   terminal length it will be split by word boundaries and printed at *
! *   several line. The maximal length of the error message is 1024      *
! *   characters.                                                        *
! *                                                                      *
! *   The last error message formatted by SOTID_ERR_LOG is kept in the   *
! *   internal butter and can be retrieved by SOTID_ERR_PEEK.            *
! *                                                                      *
! *   Comments:                                                          *
! *                                                                      *
! *   hpterm in HP-UX has a known bug: it crashes if it cannot allocate  *
! *   colors. It may occur if all available colors have been grabbed     *
! *   by another applications. Therefore, invoking SOTID_ERR_LOG can     *
! *   result in disappearing the screen. If the environment variable     *
! *   SET_COLOR is set to "NO", then SOTID_ERR_LOG will print "mmmmm"    *
! *   string with normal foreground color and hpterm will survive.       *
! *                                                                      *
! * _______________________ Modified parameters: _______________________ *
! *                                                                      *
! *     IUER ( INTEGER*4, OPT )  -- Input: mode.                         *
! *                       IUER=0  -- silent mode: to format the error    *
! *                                  message but not print it.           *
! *                       IUER=-1 -- verbose mode: to format the error   *
! *                                  message and print it in the screen  *
! *                                 Output: Error code.                  *
! *                                                                      *
! * _________________________ Input parameters:  _______________________ *
! *                                                                      *
! *     NERR ( INTEGER*4 ) -- Error code                                 *
! *     PROG ( CHARACTER, OPT ) -- Subroutine name where the error has   *
! *                                occurred.                             *
! *      MES ( CHARACTER, OPT ) -- Additional message describing the     *
! *                                error (not more than 1024 characters) *
! *                                                                      *
! * ###  28-AUG-91 SOTID_ERR_LOG  v4.0  (c) L. Petrov  07-JUN-2002  ###  *
! *                                                                      *
! ************************************************************************
        CHARACTER PROG*(*), MES*(*)
        CHARACTER  STR1*132, STR2*1024, OUT*1024, ERROR_BUFFER*1024
        COMMON   / ERROR__COMMON / ERROR_BUFFER
        EXTERNAL   SOTID_ERROR_BLKDAT
        INTEGER*4  LPRN, LUN
        PARAMETER  ( LPRN = 72 ) ! Maximal length of the output line
#ifdef HPUX
        PARAMETER  ( LUN  =  7 ) ! Logical unit number for error messages
#else
        PARAMETER  ( LUN  =  6 ) ! Logical unit number for error messages
#endif     
        INTEGER*4  LOUT, IB, LR, J1, IR, IBG, IE, IE_NEW
        INTEGER*4, EXTERNAL :: ILEN, LINDEX
!
        IF ( NERR .EQ. 0 ) THEN
!
! ---------- Error code is zero.
!
             IF ( LOC(IUER) .NE. 0 ) THEN
!
! --------------- IUER is accessible for writing.
!
                  IUER = 0
             END IF
             ERROR_BUFFER(1:3) = '???'
             RETURN
        END IF
!
! ----- Initialization
!
        CALL CLRCH ( ERROR_BUFFER )
        CALL CLRCH ( STR1 )
        CALL CLRCH ( STR2 )
!
! ----- Put there the name of the subroutine and the message itself
!
        IF ( LOC(PROG) .NE. 0 ) STR1=PROG
        IF ( LOC(MES)  .NE. 0 ) STR2=MES
!
        CALL CLRCH ( OUT )
        OUT(1:)='$$$ IUER='
        IL = ILEN(OUT)+1
        WRITE ( UNIT=OUT(IL:IL+11), FMT='(I12)' ) NERR
        CALL CHASHL ( OUT(IL:) )
!
! ----- Prepare the error buffer string
!
        ERROR_BUFFER = OUT(1:I_LEN(OUT))//' '//STR1(1:I_LEN(STR1))//' "'// &
     &                 STR2(1:I_LEN(STR2))//'"'
!
! ----- Check whether IUER is acessible wor writing
!
        IF ( LOC(IUER) .NE. 0 ) THEN
!
! ---------- Yes it is acessible. Is the input value of IUER = 0?
!
             IF ( IUER .EQ. 0 ) THEN
!
! --------------- Yes it is zero. Then return
!
                  IUER = NERR
                  RETURN
                ELSE
!
! --------------- It is not zero. Contunue
!
                  IUER = NERR
             END IF
        END IF
!
        LOUT = ILEN(ERROR_BUFFER)
        IB=1
        LR=LOUT
!
! ----- Printing it on the screen
!
        DO 410 J1=1,999
           IR = LPRN - IBG  ! acceptable lenght of one line on the screen
           IE = IB+IR  ! the last position of the symbol to be print
!
! -------- Adjusting the last position (moving back if needed)
!
           IF ( IE .GE. LOUT ) THEN
                IE=LOUT
             ELSE
!
! ------------  Seek the last blank symbol on the line.
!
                IE_NEW = LINDEX ( ERROR_BUFFER(IB:IE), ' ' ) + IB-1
                IF ( IE_NEW .GT. IB ) IE=IE_NEW  !  moving the right boundary
           END IF
           WRITE ( LUN, FMT='(A)' ) ERROR_BUFFER(IB:IE) ! writing
           IB = IE + 1
           IF ( IB .GT. LOUT ) GOTO 810
           IBG = 1
 410    CONTINUE
 810    CONTINUE
!
        RETURN
        END  SUBROUTINE  SOTID_ERR_LOG
!
! ------------------------------------------------------------------------
!
      SUBROUTINE SOTID_ERR_PEEK ( STR )
! ************************************************************************
! *                                                                      *
! *     Routine SOTID_ERR_PEEK returns the line with the latest error    *
! *   message returned by SOTID_ERR_LOG. IF SOTID_ERR_LOG was not called *
! *   at all or the last time was called as SOTID_ERR_LOG ( 0, IUER ),   *
! *   what means that no error message was to be generated, then         *
! *   SOTID_ERR_PEEK returns the empty string.                           *
! *                                                                      *
! * ________________________ Output parameters: ________________________ *
! *                                                                      *
! *  STR ( CHARACTER ) -- Last error message reported by SOTID_ERR_LOG.  *
! *                                                                      *
! *  ### 12-JUN-2002  SOTID_ERR_PEEK  v1.0 (c) L. Petrov 12-JUN-2002 ### *
! *                                                                      *
! ************************************************************************
      CHARACTER  STR*(*), ERROR_BUFFER*1024
      COMMON   / ERROR__COMMON / ERROR_BUFFER
      EXTERNAL   SOTID_ERROR_BLKDAT
!
      IF ( LOC(STR) .NE. 0 ) THEN
           CALL CLRCH ( STR )
           IF ( ERROR_BUFFER(1:3) .EQ. '???' ) THEN
                CALL CLRCH ( ERROR_BUFFER )
           END IF
           STR = ERROR_BUFFER
      END IF
!
      RETURN
      END  SUBROUTINE SOTID_ERR_PEEK
!
! ------------------------------------------------------------------------
!
      BLOCK DATA SOTID_ERROR_BLKDAT
! ************************************************************************
! *                                                                      *
! *   Block data program unit which intializes ERROR_COMMON common       *
! *   blocks. It sets only the first three fields pf ERROR_BUFFER --     *
! *   that is enough to tell to ERR_PEEK routine that ERROR_BUFFER was   *
! *   not yet properly initialized.                                      *
! *                                                                      *
! *  ### 12-JUN-2002  ERROR_BLKDAT  v1.0 (c)  L. Petrov 12-JUN-2002 ###  *
! *                                                                      *
! ************************************************************************
      CHARACTER  ERROR_BUFFER*1024
      COMMON   / ERROR__COMMON / ERROR_BUFFER
      DATA       ERROR_BUFFER(1:3) / '???' /
      END   BLOCK DATA SOTID_ERROR_BLKDAT
